% parentRots = Nx3x3 , where N is the number of parents, starting at 
% 1 = wRpelvis, 2 = pelvisRlowerback and so on.
%
% mp : 2x3  = marker positions in world coordinates for the two markers
% that make up this limb. First row = marker that joins to parent (e.g
% elbow for radius) , second row = marker that joins to child (e.g wrist for radius)
% 
function [newjaXYZ, yaw] = combinePosWithYaw(dotsHome, dotChain, mp, onlyparentRots, jaXYZnearestPose, rootTranslation)

% create wRparent
wRparent = eye(3);
for i=1:size(onlyparentRots,1)
    parentRthis = getRxyz(onlyparentRots(i,1), onlyparentRots(i,2), onlyparentRots(i,3));
    wRparent = wRparent * parentRthis;    
end

% home positions : append [0 0 0] to parentRots because current limb is in
% 'home' or 'zero' state
[mpB, mpA] = getHomePosWhileOrient([onlyparentRots; 0 0 0], dotsHome, dotChain, rootTranslation);

abworld = [(mpB - mpA)/norm(mpB-mpA)]';
abparent = wRparent' * abworld;

Rzo = createZeroOffsetMatrix(abparent);

% get jaXYZ from joint data for nearest pose
yaw = getYawMovingFrame(Rzo, jaXYZnearestPose);

% markers give us a unit vector expressed in the parent coordinate frame
mpB1 = mp(2,:); mpA1 = mp(1,:);
bworld = [(mpB1-mpA1)/norm(mpB1-mpA1)]';
bparent = wRparent' * bworld;

[rX, rY, rZ] = getJointRotUsingMarkersAndYaw(bparent, yaw, Rzo);

%disp(sprintf('combinePosWithYaw:[%.3f %.3f %.3f]',rX, rY, rZ));

newjaXYZ = [rX, rY, rZ];