% Estimate the rotation matrix that takes the home pose coordinate frame to
% the current pose coordinate frame. (see additional materials, Section 1)

function [R, angles]= estimatePelvisRotation(mp, dotsHome,skeleton,N)

mp = mp - repmat(mp((skeleton.PELVIS-1)*3+1:(skeleton.PELVIS-1)*3+3), 1,N);
dotsHome = dotsHome - repmat(dotsHome((skeleton.PELVIS-1)*3+1:(skeleton.PELVIS-1)*3+3),1,N);

% home pose, coordinate frame
h2 = dotsHome((skeleton.UPPERLEGL-1)*3+1:(skeleton.UPPERLEGL-1)*3+3) ...
    - dotsHome((skeleton.UPPERLEGR-1)*3+1:(skeleton.UPPERLEGR-1)*3+3); 
h2 = h2/norm(h2); % y axis unit vector
h3 = dotsHome((skeleton.LOWERBACK-1)*3+1:(skeleton.LOWERBACK-1)*3+3) ...
    - dotsHome((skeleton.PELVIS-1)*3+1:(skeleton.PELVIS-1)*3+3);
% z axis unit vector is the component of h3 which is perpendicular to h2
h3p = h3 - dot(h2,h3)*h2;
h3p = h3p/norm(h3p); 
h1 = cross(h2,h3p); % x axis unit vector

% current pose, coordinate frame
p3 = mp((skeleton.LOWERBACK-1)*3+1:(skeleton.LOWERBACK-1)*3+3) ...
    - mp((skeleton.PELVIS-1)*3+1:(skeleton.PELVIS-1)*3+3); 

p2 = mp((skeleton.UPPERLEGL-1)*3+1:(skeleton.UPPERLEGL-1)*3+3) ...
    - mp((skeleton.UPPERLEGR-1)*3+1:(skeleton.UPPERLEGR-1)*3+3); 
p2 = p2/norm(p2);
% z axis unit vector is the component of p3 which is perpendicular to p2
p3p = p3 - dot(p2,p3)*p2;
p3p = p3p/norm(p3p); 
p1 = cross(p2,p3p); % x axis unit vector

% The function procrustes rotation implements Equations 7 through 10 of the
% additional material.
R = procrustesrotation([h1', h2', h3'], [p1' p2' p3']);
[g,a,b,g2,b2,a2] = getRotationsXYZ(R);
angles = [g,a,b];