% Reconstruct the 3d pose that minimizes the image projection error, limb
% length error and regularization error. (see Section 3.5)
% The reconstructed 3d pose is called recon3d.
% recon2d is the projection of this pose on the image plane.
% llrecon = 3d limb lengths of the reconstructed pose
% llerrror = error in limb lengths

function [recon2d, recon3d,llrecon,llerror] = reconstructLS(match, R, c, m3d, weightll, weightreg, limblengths, result3d, skeleton, numDots)

numframes = size(match,1);

prevframe = result3d(1,:);
for i=1:numframes
    linpoint = prevframe; % We linearize the limb lengths about the previous frame. (see Section 3.5, Equation 10.)
    regpoint = result3d(i,:); % We use the motion captured frame as a regularizer. (see Section 3.5, Equation 10.)
    recon3d(i,:) = reconstructMarkers3D(match(i,:),R(1:2,:),c,m3d, weightll,...
        limblengths, linpoint, weightreg, regpoint, skeleton);
    prevframe = recon3d(i,:);
    llrecon(i,:) = getLimbLengths(recon3d(i,:), skeleton.H);
    llerror(i) = norm(llrecon(i,:)-limblengths);
end

recon2d = project3d2d(recon3d, R, numDots);